// *********************************************************************
//                       PWM output controled via RF
//                             Demo example
// *********************************************************************
//
// Intended for:
//   HW: - TR-72G + DK-EVAL-04x development kit
//
//   OS: - v4.06G
//
// Description:
//   - this example receives RF commands and controls PWM output on C5 pin.
//   - use example E03-TR as a transmiter 
//   - use IQRF IDE 4.47 or higher - Terminal (Use Numbers must be checked, see IQRF IDE Help)
//   - import prepared macros file PWM_macros.iqrfmcr to the IQRF IDE
//
//   Commands to be sent from the transmiter: 'P'<PWM duty cycle>
//      <PWM duty cycle>: 25 ~ 0.1ms, 50 ~0.2ms, ..., 225 ~ 0.9ms
//      (e.g. "P#050" -> PWM duty cycle = 0.2ms)
//
// ---------------------------------------------------------------------
// Online IQRF OS Reference Guide: https://doc.iqrf.org/IQRF-OS-Reference-guide
//
// Copyright (c) IQRF Tech s.r.o.
//
// File:    TR_PWM.c
// Version: v3.00                                   Revision: 21/06/2021
//
// Revision history:
//   v3.00: 21/06/2021  First release for TR-7xG.
//
// *********************************************************************
// Default IQRF include (modify the path according to your setup)
#include "IQRF.h"

// *********************************************************************
#define RX_FILTER               5

// *********************************************************************
void initPWM();

// *********************************************************************

void APPLICATION()
{
    setWDTon_4s();                      // WDT on, timeout 4 s (see IQRF-macros.h)
    setLEDR();
    waitDelay(50);                      // Startup indication
    stopLEDR();
    initPWM();
    setRFmode(_WPE | _RX_STD | _TX_STD);
    _checkRFcfg_PQT = 1;                // Enable preamble quality test
    toutRF = 1;                         // Wait Packed End active so the toutRF can be set to minimum
    
    while (1)
    {   
        clrwdt();
        
        if (checkRF(RX_FILTER))
        {
            if (RFRXpacket())               
            {
                if (bufferRF[0] == 'P')
                {
                    pulseLEDR();
                    CCPR3H = bufferRF[1];
                }
            }
        }
    }
}
// *********************************************************************

void initPWM()                              // CCP3 is used with C5 pin as PWM output
{
    TRISC.6 = 1;                            // C5 as input 
    TRISA.5 = 1;
    TRISB.4 = 1;

    // PPS setting
    unlockPPS();
    RB4PPS = 0x0B;                          // Out: CCP3 - RB4
    lockPPS();

    CCP3MD = 0;                             // Enable CCP3 module
    setCCPTMRS0(CCPTMRS0 | 0b00.11.00.00);  // Set timer 6 as CCP2 timer
                                            //   This register (CCPTMRS0) can not be written directly due to OS restriction
    CCP3CON = 0b1.00.1.1111;                // CCP2 ON, Left-aligned PWM mode
    CCPR3H = 25;                            // PWM duty cycle: 0.1ms
    CCPR3L = 0;
    
    TMR6MD = 0;                             // Enable TMR6 module
    T6CLKCON = 0b.0000.0001;                // Clock selection: FOSC/4
    T6CON = 0b0.100.0000;                   // Prescaler: 1:16
    T6PR = 250;                             // PWM period: 1ms @ 16 MHz CPU clock

    TMR6IF = 0;
    T6CON.7 = 1;                            // Timer 6 ON
    while (!TMR6IF);                        // To send a complete duty cycle and period on the first PWM
    TRISB.4 = 0;                            // C5 as output
}
// *********************************************************************
